#include "lmbf-gui.h"
#include "ui_lmbf-gui.h"
#include <QDebug>

LmbfGui::LmbfGui(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::LmbfGui)
{
    ui->setupUi(this);
    fftScale = new fftScaleDraw();
    ui->tPlotLightMarkerFFt->setAxisScaleDraw(QwtPlot::xBottom, fftScale);
    ui->tPlotLightMarkerFFt->axisScaleEngine(QwtPlot::xBottom)->setBase(2);

    //init internal 
    num_turns=1; //fake value for starting

    //init palettes
    normalpalette = ui->tLabel_7->palette();
    alarmpalette  = ui->tLabel_7->palette();
    normalpalette.setColor(QPalette::Background, EColor(Elettra::green));
    alarmpalette.setColor(QPalette::Background, EColor(Elettra::red));

    // create watcher to detect variations of NumTurns attribute and recalculate fft x axis fftScale

    turns_reader = new QTWatcher(this);
    turns_reader->attach(this, SLOT(update_num_turns(double)));
    turns_reader->setSource("lmbf/buf/0/numturns");
    sevr_reader = new QTWatcher(this);
    sevr_reader->attach(this, SLOT(update_sevr(QString)));
    sevr_reader->setSource("$3/IQ_STA_STATUS");
    ui->tPlotLightMarkerFFt->setSources("lmbf/buf/0/FFTADC0Avg");
} 

LmbfGui::~LmbfGui()
{
    delete turns_reader;
    delete ui;
}

void LmbfGui::update_num_turns(double nturns)
{
    qDebug() << "update_num_turns " << nturns ;
    cout << nturns <<endl;
    fftScale->set_num_turns(nturns);
    if (nturns > 0 && nturns != num_turns){
        num_turns = nturns;
        int bound= static_cast<int>(num_turns/20);
        ui->tPlotLightMarkerFFt->setXUpperBound( bound );
        ui->tPlotLightMarkerFFt->setAxisScale(QwtPlot::xBottom, 0.0, (bound), 0);
        ui->tPlotLightMarkerFFt->axisScaleEngine(QwtPlot::xBottom)->setBase(2);
    }
}


fftScaleDraw::fftScaleDraw()
{
    num_turns=1; //trick to reuse the slot during initalization
    double initial=1024.0; //guess value
    set_num_turns(initial);
}

void fftScaleDraw::set_num_turns(double n)
{
    qDebug() << "set_num_turns " << n ;
    if (n> 0 && n!=num_turns){
        num_turns = n;
    }
}

fftScaleDraw::~fftScaleDraw()
{
    fftLabels.clear();
}

QwtText fftScaleDraw::label(double v) const{
    qDebug() << "label "<<num_turns <<" "<< v << " " << (v+1)/(num_turns);
    return QString::number((v+1)/(num_turns) ,'f', 3);
}

void LmbfGui::update_sevr(QString sevr)
{
    if (sevr == "NO_ALARM"){
        ui->tLabel_7->setPalette(normalpalette);
        ui->tLabel_7->setText("ON");
    }
    else{
        ui->tLabel_7->setPalette(alarmpalette);
        ui->tLabel_7->setText("OFF / ALARM");
    }
}
